package googccwrap;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

/**
 * Wrapper class for invoking google closure compiler. This is because the compiler is based on a butchered version of
 * rhino, which ruins things when trying to use a "real" rhino from maven repo for other things.
 * 
 * @author Martin Algesten
 * 
 */
public class GoogleClosureCompilerWrapper {

    private String[] args;

    public GoogleClosureCompilerWrapper(String... args) {

        if (args == null) {
            args = new String[] {};
        }

        this.args = args;

    }

    /**
     * Compiles the given script into another string.
     * 
     * @param script
     *            The script to compile.
     * @return The compiled script as a string.
     * @throws CompilationFailedException
     *             Thrown if the compilation failed. The error message is the exception message.
     */
    public String compileString(String script) throws CompilationFailedException {

        ByteArrayInputStream bais = null;

        try {
            bais = new ByteArrayInputStream(script.getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            // as if...
        }

        InputStream is = compile(bais);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {

            Util.spool(is, baos, true);

            baos.close();

            return new String(baos.toByteArray(), "utf-8");

        } catch (IOException e) {
            throw new IllegalArgumentException("Failed to compile", e);
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                // ignore.
            }
        }

    }

    /**
     * Compiles the given input stream and returns an input stream to read the compiled contents from.
     * 
     * @param is
     *            The input stream to read (will be treated as utf-8 by closure compiler defaults).
     * @return The output stream to write (is encoded us-ascii by closure compiler defaults).
     * @throws CompilationFailedException
     *             Thrown if the compilation failed. The error message is the exception message.
     */
    public InputStream compile(InputStream is) throws CompilationFailedException {

        ClosureCompiler compiler = ClassLoaderTool.getCompiler();

        return compiler.compile(args, is);

    }
}
